#
#  AppController.rb
#  Metadata Explorer
#
#  Created by Gregory Clarke on 7/3/09.
#  Copyright (c) 2009 Intelligent Assistance, Inc. All rights reserved.
#
require 'osx/cocoa'
require 'metadata_explorer_algorithm.bundle'

class AppController < OSX::NSObject
	
	ib_outlets :mainWindow, :deactivateMenuItem, :saveMenuItem
	ib_outlets :miniMEView, :chooseXMLButton, :spinner, :xmlIcon, :xmlFileName, :filterPopup, :searchField, :buyButton, :spreadsheetTableView, :sendToFCPButton, :saveToExcelButton
	ib_outlets :progressWindow, :progressView, :progressIndicator, :progressMessage
	ib_outlets :regoView, :deactivateAreaView, :webView, :problemMessage, :closeRegoButton

	def awakeFromNib
		puts "awakeFromNib"
		check_for_expired_beta()
		@mainWindow.setTitle("Metadata Explorer")
		@xmlIcon.setHidden(true)
		@xmlIcon.setToolTip("")
		@xmlFileName.setStringValue("")
		@filterPopup.setEnabled(false)
		@sendToFCPButton.setEnabled(false)
		@highlightTextFieldCell = OSX::NSTextFieldCell.new
		@highlightTextFieldCell.setEditable(false)
		@highlightTextFieldCell.setLineBreakMode(OSX::NSLineBreakByTruncatingTail)
		@highlightTextFieldCell.setAlignment(OSX::NSRightTextAlignment)
		@highlightTextFieldCell.setTextColor(OSX::NSColor.whiteColor)
		@highlightTextFieldCell.setBackgroundColor(OSX::NSColor.colorWithDeviceRed_green_blue_alpha(0.38, 0.05, 0.11, 1))
		@highlightTextFieldCell.setDrawsBackground(true)
		# @highlightTextFieldCell.setBackgroundColor(OSX::NSColor.lightGrayColor)
		@textFieldCell = OSX::NSTextFieldCell.new
		@textFieldCell.setEditable(false)
		@textFieldCell.setLineBreakMode(OSX::NSLineBreakByTruncatingTail)
		@popup_menu_array = ["do not add", "Label 2", "Description", "Scene", "Shot/Take", "Log Note", "Master Comment 1", "Master Comment 2", "Master Comment 3", "Master Comment 4", "Comment A", "Comment B"]
		@popUpButtonCell = OSX::NSPopUpButtonCell.new
		# @popUpButtonCell.setEnabled(@this_rego[0]["active"])
		@popUpButtonCell.setBordered(false)
		@popUpButtonCell.addItemsWithTitles(@popup_menu_array)
	end

	### rego methods ###
	
	def applicationWillFinishLaunching(sender) # applicationWillFinishLaunching: is sent before application:openFile: (so it can deal with a file dropped onto the app icon)
		puts "applicationWillFinishLaunching"
		check_for_expired_beta()
		if @this_rego[0]["active"]
			showMiniMEView()
		elsif @this_rego[0]["mac_problem"]==true
			check_for_rego(@this_rego)
		else
			@deactivateMenuItem.setTitle("Activate...")
			displayDemoSheet()
		end
	end
	
	def displayDemoSheet
		report = OSX::NSAlert.alloc.init
		report.setMessageText("#{SKU} has not been activated \nfor this machine.")
		report.setAlertStyle(OSX::NSInformationalAlertStyle)
		report.setInformativeText("Would you like to activate #{SKU}, or use it in free mode?")
		report.addButtonWithTitle("Activate")
		report.addButtonWithTitle("Free")
		report.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, "didEndDemoSheet:returnCode:contextInfo:", nil)
	end

	def didEndDemoSheet_returnCode_contextInfo(sheet, buttonClicked, info)
		sheet.window.orderOut(nil)
		if buttonClicked==OSX::NSAlertFirstButtonReturn
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		else
			@buyButton.setHidden(false)
			showMiniMEView()
		end
	end

	def showDeactivation
		if @deactivateMenuItem.title=="Activate..."
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		else
			newFrame = self.newFrameForNewContentView(@deactivateAreaView) # 500 x 200
			@mainWindow.setMinSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setMaxSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setContentView(@deactivateAreaView)
			showActivationView(newFrame, WEB_APP + "destroy/" + @this_rego[0]["number"].to_s)
		end
		@closeRegoButton.setTitle("Cancel")
	end
	ib_action :showDeactivation # Metadata Explorer > Activate or Deactivate
	
	def showActivationView(frame, url)
		puts "showActivationView #{url}"
		@mainWindow.setFrame_display frame, true
		@mainWindow.setContentView(@regoView)
		@mainWindow.setTitle("Activation")
		# notification
		center = OSX::NSNotificationCenter.defaultCenter
		center.addObserver_selector_name_object(self, :webViewProgressFinished, "WebProgressFinishedNotification", @webView) # name is WebProgressFinishedNotification NOT WebViewProgressFinishedNotification
		@webView.mainFrame.loadRequest( OSX::NSURLRequest.requestWithURL( OSX::NSURL.URLWithString(url) ) )
	end
	
	def webViewProgressFinished(notification)
		puts notification.name
		if notification.name=="WebProgressFinishedNotification"
			if @webView.mainFrame.dataSource.nil? # in case page doesn't load...
				@webView.mainFrame.stopLoading
				@problemMessage.setHidden(false)
				@webView.setHidden(true)
			else
				get_web_app_response(@webView.mainFrame.dataSource.response.URL)
			end
		end
	end 
	
	def closeActivationView
		if @closeRegoButton.title=="Continue" || @closeRegoButton.title=="Cancel"
			setupView(@miniMEView, "Metadata Explorer", 725, 400)
		else
			OSX::NSApplication.sharedApplication.terminate(self)
		end
	end
	ib_action :closeActivationView # on click of Registration window's Continue button
	
	### end rego methods ###
	
	def showMiniMEView
		puts "showMiniMEView"
		@mainWindow.setContentView(@miniMEView)
	end

	def setupView(view, title, width, height)
		@mainWindow.setTitle(title)
		oldFrame = @mainWindow.frame
		newFrame = self.newFrameForNewContentView(view)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setFrame_display_animate(newFrame, true, true)
		@mainWindow.setContentView(view)
	end

	def newFrameForNewContentView(view)
		newFrameRect = @mainWindow.frameRectForContentRect(view.frame)
		oldFrameRect = @mainWindow.frame
		newSize = newFrameRect.size
		oldSize = oldFrameRect.size
		frame = @mainWindow.frame
		frame.size = newSize
		frame.origin.y = frame.origin.y - (newSize.height - oldSize.height)
		frame
	end

	def showHelp(sender)
		openPDF("Metadata Explorer - Help.pdf")
	end
	ib_action :showHelp # Help > Metadata Explorer Help
	
	def openPDF(filename)
		path_to_file = __FILE__.sub("AppController.rb",filename)
		system('open', path_to_file)
	end

	def visitBuyPage(sender)
		system('open', 'http://www.AssistedEditing.com/miniME/buy')
	end
	ib_action :visitBuyPage # Help > Visit www.AssistedEditing.com

	def visitWebsite(sender)
		system('open', 'http://www.AssistedEditing.com/')
	end
	ib_action :visitWebsite # Help > Visit www.AssistedEditing.com

	def sendFeedback(sender)
		OSX::JRFeedbackController.showFeedback
	end
	ib_action :sendFeedback # Metadata Explorer > Provide Feedback

	def chooseXmlFile(sender)
		@xmlIcon.setHidden(true)
		@xmlIcon.setToolTip("")
		@xmlFileName.setStringValue("")
		filetypes = ["xml"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setTitle("Choose the exported Bin(s) of Clips to process (XML file)")
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndOpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseXmlFile

	def didEndOpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			file = oPanel.filenames.objectAtIndex(0)
			application_openFile(self, file)
		else
			@xmlIcon.setHidden(true)
			@xmlIcon.setToolTip("")
			@xmlFileName.setStringValue("")
		end
	end

	def application_openFile(sender, path)
		OSX::NSDocumentController.sharedDocumentController().noteNewRecentDocumentURL( OSX::NSURL.fileURLWithPath(path) ) # add this file to the Recent menu
		# fileManager = OSX::NSFileManager.defaultManager
		chosen_file = path.to_s
		chosen_file_extension = File.extname(chosen_file)
		@xmlIcon.setHidden(false)
		@xmlIcon.setToolTip(path.to_s)
		@xmlFileName.setStringValue(File.basename(chosen_file))
		@xmlIcon.setNeedsDisplay(true)
		@xmlFileName.setNeedsDisplay(true)
		@filterPopup.setEnabled(false)
		begin
			@progressIndicator.setIndeterminate(true)
			@progressIndicator.setUsesThreadedAnimation(true)
			@progressIndicator.startAnimation(self)
			displayProgressSheet()
			explore_metadata(chosen_file)
			if @clips_metadata.empty?
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was no QuickTime or Exif metadata in this XML file.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
				chooseXmlFile(nil)
			else
				load_tableView(self)
				@filterPopup.setEnabled(true)
				@saveMenuItem.setEnabled(true)
				@saveToExcelButton.setEnabled(true)
			end
		rescue
			didEndProgressSheet_returnCode_contextInfo(@progressWindow, -1, nil)
			alert = OSX::NSAlert.alloc.init
			alert.setMessageText("There was a problem with this XML file.")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			puts "RESCUE!!! Problem with explore_metadata()"
			alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
			alert.addButtonWithTitle("OK")
			alert.runModal
			chooseXmlFile(nil)
		end
	end

	def displayProgressSheet
		OSX::NSApp.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo(@progressWindow, @mainWindow, self, 'didEndProgressSheet:returnCode:contextInfo:', nil)
		@progressView.displayIfNeeded
	end

	def didEndProgressSheet_returnCode_contextInfo(sheet, code, info)
		puts "didEndProgressSheet_returnCode_contextInfo code #{code}"
		sheet.orderOut(nil) # or self?
		@progressMessage.setStringValue("")
		@progressIndicator.stopAnimation(self)
		OSX::NSApp.endSheet(sheet)
		if code < 0 # error condition
			@saveMenuItem.setEnabled(false)
			@saveToExcelButton.setEnabled(false)
		else
			@saveMenuItem.setEnabled(true)
			@saveToExcelButton.setEnabled(true)
		end
	end

	# save temporary XML file and open into FCP (no need for an open sheet)
	def saveXml
		@progressIndicator.setIndeterminate(true)
		@progressIndicator.setUsesThreadedAnimation(true)
		@progressIndicator.startAnimation(self)
		displayProgressSheet()
		# begin
			add_fcp_log_notes()
		# rescue
		# 	didEndProgressSheet_returnCode_contextInfo(@progressWindow, -1, nil)
		# 	alert = OSX::NSAlert.alloc.init
		# 	alert.setMessageText("There was a problem sending this metadata to Final Cut Pro.")
		# 	alert.setAlertStyle(OSX::NSInformationalAlertStyle)
		# 	puts "RESCUE!!! Problem with saveXml()"
		# 	alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
		# 	alert.addButtonWithTitle("OK")
		# 	alert.runModal
		# end
	end
	ib_action :saveXml

	def saveExcel
		sPanel = OSX::NSSavePanel.savePanel
		sPanel.setTitle("Save the Excel spreadsheet")
		sPanel.setRequiredFileType("xls")
		sPanel.setExtensionHidden(false)
		sPanel.setCanSelectHiddenExtension(true)
		sPanel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, @mainWindow, self, 'didEndSaveExcel:returnCode:contextInfo:', nil)
	end
	ib_action :saveExcel
	
	def didEndSaveExcel_returnCode_contextInfo(sPanel, buttonClicked, info)
		@spinner.startAnimation(self)
		########## MAKE AND SAVE EXCEL ##########
		if buttonClicked == OSX::NSOKButton
			sPanel.orderOut(self)
			if sPanel.isExtensionHidden?
				extension_hidden = 1
			else
				extension_hidden = 0
			end
			puts "is extension hidden?", sPanel.isExtensionHidden
			excel_file = sPanel.filenames.objectAtIndex(0).to_s
			begin
				save_to_excel(excel_file)
				OSX::NSWorkspace.sharedWorkspace.openFile(excel_file)
			rescue
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was a problem saving an Excel spreadsheet of this metadata.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				puts "RESCUE!!! Problem with saveExcel()"
				alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
			end
		end
		@spinner.stopAnimation(self)
	end
	
	def applicationShouldTerminateAfterLastWindowClosed(sender)
		true
	end

	### TableView methods ###

	def load_tableView(sender) # re-builds @clips_in_spreadsheet and @column_headings
		filtered_qt_column_headings = @qt_column_headings
		filtered_exif_column_headings = @exif_column_headings
		filter_string = @searchField.stringValue.to_s.downcase
		unless filter_string.empty?
			filtered_qt_column_headings = @qt_column_headings.find_all { |h| h.downcase.index(filter_string)!=nil }
			filtered_exif_column_headings = @exif_column_headings.find_all { |h| h.downcase.index(filter_string)!=nil }
		end
		case @filterPopup.titleOfSelectedItem
		when "All"
			@column_headings = ["Clip Name", "File Name"] + filtered_qt_column_headings + filtered_exif_column_headings
			@clips_in_spreadsheet = [@first_row] + @clips_metadata # data model for the NSTableView
		when "QuickTime"
			@column_headings = ["Clip Name", "File Name"] + filtered_qt_column_headings
			@clips_in_spreadsheet = [@first_row] + @clips_metadata.find_all { |c| c["Type"]=="QuickTime" }
		when "Exif"
			@column_headings = ["Clip Name", "File Name"] + filtered_exif_column_headings
			@clips_in_spreadsheet = [@first_row] + @clips_metadata.find_all { |c| c["Type"]=="Exif" }
		end
		# create or hide the columns and header cells
		@spreadsheetTableView.tableColumns.each_with_index do |tableColumn, i|
			if i < @column_headings.length # set column heading
				tableColumn.headerCell.setStringValue(@column_headings[i])
				tableColumn.setHeaderToolTip(@column_headings[i])
				tableColumn.setHidden(false)
				tableColumn.sizeToFit unless (i==0 || i==1)
				tableColumn.setMinWidth(100)
			else # empty
				tableColumn.headerCell.setStringValue("")
				tableColumn.setHeaderToolTip("")
				tableColumn.setHidden(true)
			end
		end
		@spreadsheetTableView.reloadData
		@spreadsheetTableView.deselectAll(self)
	end
	ib_action :load_tableView

	def numberOfRowsInTableView(tableView)
		if @clips_in_spreadsheet.nil?
			return 0
		else
			return @clips_in_spreadsheet.length
		end
	end

	def tableView_objectValueForTableColumn_row(tableView, tableColumn, rowIndex)
		# puts "tableView_objectValueForTableColumn_row"
		@column_headings.each_with_index do |column, i|
			# unless tableColumn.nil?
			if tableColumn.headerCell.stringValue==column
				object = @clips_in_spreadsheet[rowIndex]
				return object[column]
			end
		end
	end
	
	def tableView_dataCellForTableColumn_row(tableView, tableColumn, rowIndex)
		# puts "tableView_dataCellForTableColumn_row"
		tableView.deselectRow(0)
		unless tableColumn.nil?
			if rowIndex==0
				column_header = tableColumn.headerCell.stringValue.to_s
				if column_header=="Clip Name"
					return @textFieldCell
				elsif column_header=="File Name"
					return @highlightTextFieldCell
				else
					return @popUpButtonCell
				end
			else
				return @textFieldCell
			end
		end
	end
	
	def tableView_setObjectValue_forTableColumn_row(tableView, anObject, tableColumn, rowIndex)
		if rowIndex==0
			column_header = tableColumn.headerCell.stringValue.to_s
			@clips_in_spreadsheet[0][column_header] = anObject.to_i
		end
	end

end
